import logging
import os
from datetime import datetime
from typing import Optional

import requests
from payments.types import BankAccountData, CreditCard, CreditCardHolderInfo
from requests.exceptions import RequestException, Timeout

logger = logging.getLogger("celery")


class AsaasClient:
    BASE_URL = os.getenv("ASAAS_API_URL", "https://api-sandbox.asaas.com")

    def __init__(self, version: str = "v3", *args, **kwargs):
        self.version = version
        self.token = os.getenv("ASAAS_ROOT_API_KEY")
        self.headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "access_token": self.token,
        }

    def create_customer_account(
        self, name: str, cpfCnpj: str, phone_number: str
    ) -> Optional[str]:
        """Creates a customer on Asaas and return their ID if successful."""
        url = f"{self.BASE_URL}/{self.version}/customers"

        payload = {"name": name, "cpfCnpj": cpfCnpj, "phone": phone_number}

        try:
            response = requests.post(url, json=payload, headers=self.headers, timeout=4)
            response.raise_for_status()
            data = response.json()
            return data.get("id")
        except (RequestException, Timeout):
            logger.error("Erro ao criar cliente na Asaas API", exc_info=True)
            return None

    def get_customer_account(self, customer_id: str) -> Optional[dict]:
        """
        Fetch a customer from Asaas by ID.
        Returns a dict with id, name, cpfCnpj, and phone if found; otherwise None.
        """
        url = f"{self.BASE_URL}/{self.version}/customers/{customer_id}"

        try:
            response = requests.get(url, headers=self.headers, timeout=4)
            response.raise_for_status()
            data = response.json()

            if not data or "id" not in data:
                return {}

            return {
                "id": data.get("id"),
                "name": data.get("name"),
                "cpfCnpj": data.get("cpfCnpj"),
                "phone": data.get("phone"),
            }
        except (RequestException, Timeout):
            logger.error("Erro ao buscar cliente na Asaas API", exc_info=True)
            return {}

    def update_customer_account(self, id: str, cpfCnpj: str, mobile_phone: str) -> None:
        """
        Updates a customer on Asaas using their ID.
        """
        url = f"{self.BASE_URL}/{self.version}/customers/{id}"
        payload = {
            "cpfCnpj": cpfCnpj,
            "phone": mobile_phone,
        }

        try:
            response = requests.post(url, json=payload, headers=self.headers, timeout=4)
            response.raise_for_status()
        except (RequestException, Timeout):
            logger.error(f"Erro ao atualizar cliente {id} na Asaas API", exc_info=True)

    def delete_customer_account(self, id: str) -> Optional[dict]:
        """
        Deletes a customer from Asaas by ID.
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/customers/{id}"

        try:
            response = requests.delete(url, headers=self.headers, timeout=4)
            response.raise_for_status()
            return response.json()
        except (RequestException, Timeout):
            logger.error(f"Erro ao deletar cliente {id} na Asaas API", exc_info=True)
            return None

    def create_professional_account(
        self,
        name: str,
        email: str,
        cpf_cnpj: str,
        mobile_phone: str,
        income_value: float,
        address: str,
        address_number: str,
        province: str,
        postal_code: str,
        company_type: str,
    ) -> Optional[dict]:
        """
        Creates a white label sub-account for a professional.
        Returns a dict with the apiKey for the account and unique walletId for payment splits
        or None if the request failed.
        """
        url = f"{self.BASE_URL}/{self.version}/accounts"

        payload = {
            "name": name,
            "email": email,
            "cpfCnpj": cpf_cnpj,
            "mobilePhone": mobile_phone,
            "incomeValue": income_value,
            "address": address,
            "addressNumber": address_number,
            "province": province,
            "postalCode": postal_code,
            "companyType": company_type,
        }

        try:
            response = requests.post(url, json=payload, headers=self.headers, timeout=4)
            response.raise_for_status()
            data = response.json()

            return {
                "api_key": data.get("apiKey"),
                "wallet_id": data.get("walletId"),
            }

        except (RequestException, Timeout):
            logger.error(
                "Erro ao criar subconta white label na Asaas API", exc_info=True
            )
            return None

    def get_professional_account(self, subaccount_id: str) -> Optional[dict]:
        """
        Retrieves information about a white label sub-account given its id.
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/accounts/{subaccount_id}"

        try:
            response = requests.get(url, headers=self.headers, timeout=4)
            response.raise_for_status()
            return response.json()
        except (RequestException, Timeout):
            logger.error(
                f"Erro ao recuperar informações da subconta {subaccount_id}",
                exc_info=True,
            )
            return None

    def delete_professional_account(self, api_key: str) -> Optional[dict]:
        """
        Deletes a white label subaccount.
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/myAccount"

        self.token = api_key

        try:
            response = requests.delete(url, headers=self.headers, timeout=4)
            response.raise_for_status()
            return response.json()
        except (RequestException, Timeout):
            logger.error("Erro ao deletar profissional na Asaas API", exc_info=True)
            return None

    def request_withdrawal_TED(
        self, api_key: str, value: float, bank_account: BankAccountData
    ) -> Optional[dict]:
        """
        Requests a withdrawal for a white label sub-account via TED transfer.
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/transfers"

        self.token = api_key
        payload = {
            "value": value,
            "bankAccount": bank_account,
            "operationType": "TED",
        }

        try:
            response = requests.post(url, json=payload, headers=self.headers, timeout=4)
            response.raise_for_status()
            return response.json()

        except (RequestException, Timeout):
            logger.error("Erro ao solicitar saque via TED", exc_info=True)
            return None

    def request_withdrawal_PIX(
        self, api_key: str, value: float, pix_address_key: str, pix_key_type: str
    ) -> Optional[dict]:
        """
        Requests a withdrawal for a white label sub-account via PIX.
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/transfers"
        self.token = api_key

        payload = {
            "value": value,
            "pixAddressKey": pix_address_key,
            "pixAddressKeyType": pix_key_type,
            "operationType": "PIX",
        }

        try:
            response = requests.post(url, json=payload, headers=self.headers, timeout=4)
            response.raise_for_status()
            return response.json()

        except (RequestException, Timeout):
            logger.error("Erro ao solicitar saque via PIX", exc_info=True)
            return None

    def get_professional_account_docs(self, api_key: str) -> Optional[dict]:
        """
        Retrieves information about the documents sent or to be sent for the subaccount to be fully approved by Asaas.
        It also includes the onboardingUrl for pending documents.
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/myAccount/documents"
        self.token = api_key

        try:
            response = requests.get(url, headers=self.headers, timeout=4)
            response.raise_for_status()
            return response.json()

        except (RequestException, Timeout):
            logger.error(
                "Erro ao consultar documentos da subconta profissional", exc_info=True
            )

    def create_key_EVP(self, api_key: str) -> Optional[dict]:
        """
        Creates a new random PIX key (EVP) for the professional (white label account).
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/pix/addressKeys"
        self.token = api_key

        # the payload will be the same for every request,
        # since Asaas can only create EVP keys via API at the moment.
        payload = {"type": "EVP"}

        try:
            response = requests.post(url, headers=self.headers, json=payload, timeout=4)
            response.raise_for_status()
            return response.json()
        except (RequestException, Timeout):
            logger.error("Erro ao criar chave EVP na Asaas API", exc_info=True)
            return None

    def list_active_EVP_keys(self, api_key: str) -> Optional[dict]:
        """
        List all active PIX EVP keys for the professional (white label account).
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/pix/addressKeys?status=ACTIVE"
        self.token = api_key

        try:
            response = requests.post(url, headers=self.headers, timeout=4)
            response.raise_for_status()
            return response.json()
        except (RequestException, Timeout):
            logger.error("Erro ao listar chaves EVP na Asaas API", exc_info=True)
            return None

    def create_payment_PIX(
        self, customer_id: str, value: float, dueDate: datetime
    ) -> Optional[dict]:
        """
        Creates a new PIX payment.
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/payments"

        payload = {
            "customer": customer_id,
            "billingType": "PIX",
            "value": value,
            "dueDate": dueDate,
        }

        try:
            response = requests.post(url, headers=self.headers, json=payload, timeout=4)
            response.raise_for_status()
            return response.json()
        except (RequestException, Timeout):
            logger.error("Erro ao criar cobrança PIX na Asaas API", exc_info=True)
            return None

    def create_payment_credit_card(
        self,
        customer_id: str,
        value: float,
        dueDate: datetime,
        creditCard: CreditCard,
        creditCardHolderInfo: CreditCardHolderInfo,
    ) -> Optional[dict]:
        """
        Creates a new credit card payment.
        Returns the JSON response from Asaas, or None if failed.
        """
        url = f"{self.BASE_URL}/{self.version}/payments"

        payload = {
            "customer": customer_id,
            "value": value,
            "dueDate": dueDate,
            "creditCard": creditCard,
            "creditCardHolderInfo": creditCardHolderInfo,
        }

        try:
            response = requests.post(url, headers=self.headers, json=payload, timeout=4)
            response.raise_for_status()
            return response.json()
        except (RequestException, Timeout):
            logger.error(
                "Erro ao criar cobrança cartão de crédito na Asaas API", exc_info=True
            )
            return None



from django.core import serializers
from django.utils.functional import cached_property

from lacrei_models.core.lacreiid.adapters import AccountAdapter
from lacrei_models.core.lacreisaude.template_context_models import (
    BoardVerificationNumberRejectedContext,
    PostRegistrationApprovedContext,
    PostRegistrationRejectedContext,
)
from lacrei_models.core.notification.signals import notification
from lacrei_models.core.utils.sites import format_professional_url

from .constants import (
    BOARD_VERIFICATION_NOT_APPROVED_YET,
    PROFESSIONAL_ALREADY_APPROVED_FOR_STEP,
    PROFESSIONAL_VALIDATION_PENDING_ADMIN,
    PROFESSIONAL_VALIDATION_PENDING_DATA_INPUT,
)
from .exceptions import CannotSetStatusError


class VerificationStep:
    name: str
    url_path_to_submit: str
    url_path_to_wait_completion: str
    description: str

    def __init__(self, professional):
        self.professional = professional

    @cached_property
    def is_completed(self):  # pragma: no cover
        raise NotImplementedError

    @cached_property
    def is_submitted(self):  # pragma: no cover
        raise NotImplementedError

    @cached_property
    def redirect_to(self):
        if self.is_completed:
            return None

        if self.is_submitted:
            return self.url_path_to_wait_completion

        return self.url_path_to_submit

    def to_representation(self):
        return {
            "is_completed": self.is_completed,
            "is_submitted": self.is_submitted,
            "redirect_to": self.redirect_to,
            "description": self.description,
            "internal_message": (
                PROFESSIONAL_VALIDATION_PENDING_DATA_INPUT
                if not self.is_submitted
                else PROFESSIONAL_VALIDATION_PENDING_ADMIN
            ),
        }

    def set_approved(self, professsional_review, request=None):  # pragma: no cover
        raise NotImplementedError

    def set_rejected(self, professsional_review, request=None):  # pragma: no cover
        raise NotImplementedError


class BoardRegistrationNumber(VerificationStep):
    """
    Uma pessoa interna da lacrei irá validar o número do conselho no respectivo portal."

    Após validação, será enviado um email de confirmação, onde:
    - Se rejeitado: apagamos o cadastro e pedimos para refazer, com um número certo.
    - Se aprovado: segue para o pós cadastro.
    """

    name = "board_registration_number"
    url_path_to_submit = "/"
    url_path_to_wait_completion = "/saude/verificacao-inscricao/"
    description = "1ª - Validação no conselho"

    @cached_property
    def is_completed(self):
        return self.professional.profile_status == "approved"

    @cached_property
    def is_submitted(self):
        return True

    def set_approved(self, professsional_review, request=None):
        if self.professional.profile_status == "approved":
            raise CannotSetStatusError(PROFESSIONAL_ALREADY_APPROVED_FOR_STEP)

        self.professional.profile_status = "approved"
        self.professional.save(update_fields=["profile_status"])
        AccountAdapter().send_confirmation_professional(self.professional.user, request)

    def set_rejected(self, professional_review, request=None):
        if self.professional.profile_status == "approved":
            raise CannotSetStatusError(PROFESSIONAL_ALREADY_APPROVED_FOR_STEP)

        user = self.professional.user
        email = user.email

        professional_review.rejected_professional_data = {
            "name": professional_review.professional_name,
            "profession": self.professional.profession.name,
            "state": self.professional.state.code,
            "board_registration_number": self.professional.board_registration_number,
            "email": email,
        }
        email_context = BoardVerificationNumberRejectedContext().model_dump(mode="json")
        user.profile.delete()
        self.professional.delete()
        user.delete()
        professional_review.professional = None
        professional_review.save()

        notification.send(
            sender=self,
            template_prefix="verification/board_verification_number_rejected",
            email=email,
            context=email_context,
            recipient=None,
        )


class EmailConfirmation(VerificationStep):
    """
    Nessa etapa, uma pessoa interna da Lacrei já aprovou o cadastro,
    e agora aguardamos a pessoa profissional confirmar o email.
    """

    name = "email_confirmation"
    url_path_to_submit = "/"
    url_path_to_wait_completion = "/saude/verificacao-inscricao/"
    description = "2ª - Confirmação do email"

    @cached_property
    def is_completed(self):
        return self.professional.user.email_verified

    @cached_property
    def is_submitted(self):
        return False


class PostRegistrationData(VerificationStep):
    """
    Nessa passo a pessoa profissional irá enviar os dados restantes para o cadastro.

    Após validação, será enviado um email de confirmação, onde:
    - Se rejeitado: mandamos email pedindo para reenviar os dados.
    - Se aprovado: Liberado para ficar ativo na plataforma
    """

    name = "post_registration_data"
    url_path_to_submit = "/saude/cadastro-dados-pessoais/"
    url_path_to_wait_completion = "/saude/painel-cadastro-analise/"
    description = "4ª - Pós cadastro"

    @cached_property
    def is_completed(self):
        return self.professional.active

    @cached_property
    def is_submitted(self):
        return hasattr(self.professional, "clinic") and bool(
            self.professional.board_certification_selfie
        )

    def set_approved(self, professsional_review, request=None):
        if self.is_completed:
            raise CannotSetStatusError(PROFESSIONAL_ALREADY_APPROVED_FOR_STEP)

        if not BoardRegistrationNumber(self.professional).is_completed:
            raise CannotSetStatusError(BOARD_VERIFICATION_NOT_APPROVED_YET)

        self.professional.active = True
        self.professional.save(update_fields=["active"])

        email_context = PostRegistrationApprovedContext(
            button_url=format_professional_url(""),
        ).model_dump(mode="json")
        notification.send(
            sender=self,
            template_prefix="verification/post_registration_approved",
            email=self.professional.user.email,
            context=email_context,
            recipient=self.professional.user,
        )

    def set_rejected(self, professsional_review, request=None):
        # Save clinic data
        clinic_data = serializers.serialize("json", [self.professional.clinic])
        professsional_review.rejected_professional_data = clinic_data
        professsional_review.save()

        # Delete clinic data
        self.professional.clinic.delete()

        email_context = PostRegistrationRejectedContext(
            button_url=format_professional_url(""),
        ).model_dump(mode="json")
        notification.send(
            sender=self,
            template_prefix="verification/post_registration_rejected",
            email=self.professional.user.email,
            context=email_context,
            recipient=self.professional.user,
        )


class IntersectionalityData(VerificationStep):
    """
    Na etapa, pedimos que adicione os dados de interseccionalidade.
    """

    name = "add_intersectionality_data"
    url_path_to_submit = "/saude/cadastro-diversidade/"
    url_path_to_wait_completion = ""
    description = "3ª - Diversidade"

    @cached_property
    def is_completed(self):
        professional = self.professional
        intersectionality_fields = {
            "ethnic_group": bool(
                professional.ethnic_group_id or professional.other_ethnic_group
            ),
            "pronoun": bool(professional.pronoun_id or professional.other_pronoun),
            "sexual_orientation": bool(
                professional.sexual_orientation_id
                or professional.other_sexual_orientation
            ),
            "gender_identity": bool(
                professional.gender_identity_id or professional.other_gender_identity
            ),
        }
        return all(intersectionality_fields.values())

    @cached_property
    def is_submitted(self):
        return self.is_completed


class VerificationStepsService:
    def __init__(self, professional):
        self.professional = professional
        self.steps = [
            BoardRegistrationNumber(professional),
            EmailConfirmation(professional),
            IntersectionalityData(professional),
            PostRegistrationData(professional),
        ]

    def to_representation(self) -> dict:
        current_step = None
        steps_repr = {}

        for step in self.steps:
            if not step.is_completed and not current_step:
                current_step = step.name

            steps_repr[step.name] = step.to_representation()

        return {"current_step": current_step, "steps": steps_repr}

    def current_step(self):
        representation = self.to_representation()
        if not representation["current_step"]:
            return None
        return representation["steps"][representation["current_step"]]



import calendar
from collections import defaultdict
from datetime import datetime, timedelta
from decimal import Decimal
from random import randint

from appointments.models import Appointment
from django.conf import settings
from django.core.validators import MinValueValidator
from django.db import models
from django.urls import reverse
from django.utils import timezone
from django.utils.translation import gettext as _
from lacreiid.models import NULLABLE, BaseProfile
from lacreisaude.constants import CONTACT_REQUEST_SMS_MESSAGE
from lacreisaude.managers import ContactRequestManager
from lacreisaude.services.verification import (
    BoardRegistrationNumber,
    PostRegistrationData,
    VerificationStepsService,
)
from lacreisaude.template_context_models import NewComplaintContext
from notification.signals import notification
from phonenumber_field.modelfields import PhoneNumberField
from utils.models import BaseModel, HashedAutoField, HashedFileName
from utils.validators import BRCPFValidator, OnlyAlphabeticValidator
from watson import search as watson

PROFILE_STATUS = [
    ("pending", _("Informações pendentes")),
    ("in_review", _("Em revisão")),
    ("rejected", _("Rejeitado")),
    ("approved", _("Aprovado")),
]

VERIFICATION_STEPS_MAP = {
    "board_registration_number": {
        "choice_description": "1º etapa - Numero de inscrição",
        "verification_step": BoardRegistrationNumber,
    },
    "post_registration_data": {
        "choice_description": "2º etapa - Pós cadastro",
        "verification_step": PostRegistrationData,
    },
}
VERIFICATION_STEPS_CHOICES = (
    (key, value["choice_description"]) for key, value in VERIFICATION_STEPS_MAP.items()
)


class Profession(BaseModel):
    name = models.CharField(max_length=100)
    search_synonym = models.CharField(max_length=256, default=None, **NULLABLE)

    class Meta:
        verbose_name = _("Profissão")
        verbose_name_plural = _("Profissões")
        app_label = "lacreisaude"

    def __str__(self):
        return self.name


class WaitingList(BaseModel):
    email = models.EmailField()
    profession = models.ForeignKey(Profession, on_delete=models.PROTECT)
    state = models.ForeignKey("address.State", on_delete=models.PROTECT)

    class Meta:
        verbose_name = _("Lista de espera")
        verbose_name_plural = _("Listas de espera")
        app_label = "lacreisaude"


class Professional(BaseProfile):
    CPF = "CPF"
    CNPJ = "CNPJ"
    EMAIL = "EMAIL"
    CELULAR = "PHONE"
    EVP = "EVP"
    tipo_chave = [
        (CPF, _("CPF")),
        (CNPJ, _("CNPJ")),
        (EMAIL, _("EMAIL")),
        (CELULAR, _("PHONE")),
        (EVP, _("EVP")),
    ]
    id = HashedAutoField(primary_key=True)
    user = models.OneToOneField("lacreiid.User", on_delete=models.PROTECT)
    full_name = models.CharField(
        max_length=200,
        verbose_name=_("Nome completo"),
        validators=[OnlyAlphabeticValidator()],
    )
    about_me = models.TextField(**NULLABLE, verbose_name=_("Sobre mim"))
    profile_status = models.CharField(
        max_length=30,
        choices=PROFILE_STATUS,
        default="pending",
        verbose_name=_("Status do perfil"),
    )
    state = models.ForeignKey(
        "address.State", on_delete=models.PROTECT, verbose_name=_("Estado")
    )
    active = models.BooleanField(
        default=False,
        help_text=(
            _("Define se o perfil está ativo e pode ser alterado pelos administradores")
        ),
        verbose_name=_("Ativo"),
    )
    published = models.BooleanField(
        default=True,
        help_text=(
            _(
                "Define se o perfil está publicado "
                "e pode ser alterado pelo próprio profissional"
            )
        ),
        verbose_name=_("Publicado"),
    )
    document_number = models.CharField(
        max_length=20, **NULLABLE, validators=[BRCPFValidator()], verbose_name=_("CPF")
    )
    profession = models.ForeignKey(
        Profession,
        related_name="professionals",
        on_delete=models.PROTECT,
        verbose_name=_("Profissão"),
    )
    search_synonym = models.CharField(max_length=256, default=None, **NULLABLE)
    board_registration_number = models.CharField(
        max_length=100, verbose_name=_("Número do registro do conselho")
    )
    accepted_privacy_document = models.BooleanField(
        default=False, verbose_name=_("Documento de privacidade aceito")
    )
    privacy_document = models.ForeignKey(
        "lacreiid.PrivacyDocument",
        on_delete=models.PROTECT,
        null=True,
        blank=False,
        verbose_name=_("Documento de privacidade"),
    )
    safety_measures = models.CharField(
        max_length=1000,
        **NULLABLE,
        help_text=_("Medidas de segurança do Covid, por exemplo"),
        verbose_name=_("Medidas de segurança"),
    )
    specialty = models.CharField(
        **NULLABLE,
        max_length=250,
        help_text=_("Especialidade da pessoa profissional, como cardiologia"),
        verbose_name=_("Especialidade clínica"),
    )
    specialty_number_rqe = models.CharField(
        **NULLABLE,
        max_length=10,
        help_text=_("Registro de Qualificação de Especialidade (RQE)"),
        verbose_name=_("Registro de Qualificação de Especialidade (RQE)"),
    )
    board_certification_selfie = models.ImageField(
        **NULLABLE,
        upload_to=HashedFileName("board_certification_selfie"),
        verbose_name=_("Selfie do documento de registro do conselho"),
    )
    photo = models.ImageField(
        **NULLABLE,
        upload_to=HashedFileName("professional_photos"),
        verbose_name=_("Foto de perfil"),
    )
    photo_description = models.CharField(
        **NULLABLE,
        max_length=250,
        help_text=_("Descrição da foto de perfil para pessoas com deficiência visual"),
        verbose_name=_("Descrição da foto de perfil"),
    )
    google_calendar_credentials_json = models.JSONField(
        null=True,
        blank=True,
        help_text="Credenciais OAuth 2.0 para a API do Google Calendar.",
    )
    chave_pix = models.CharField(max_length=100, default=None, **NULLABLE)
    tipo_chave_pix = models.CharField(
        max_length=10,
        choices=tipo_chave,
        default=None,
        null=True,
        blank=True,
        verbose_name=_("tipo de chave pix"),
    )
    income_value = models.DecimalField(
        max_digits=10,
        decimal_places=2,
        **NULLABLE,
        verbose_name=_("Renda/Faturamento Mensal"),
        help_text=_(
            "Renda mensal para PF ou faturamento mensal para PJ (obrigatório para criação)"
        ),
    )

    class Meta:
        verbose_name = _("Profissional")
        verbose_name_plural = _("Profissionais")
        app_label = "lacreisaude"

    def __str__(self):
        return self.full_name

    @property
    def current_step(self):
        current_step = VerificationStepsService(self).current_step()
        if not current_step:
            return None
        return current_step["description"], current_step["internal_message"]

    def get_all_times(self, type, duration):
        all_schedules = self.clinic.opening_schedules.filter(schedule_type=type)
        all_times = defaultdict(list)

        SHORT_TO_INDEX = {
            "mon": 0,
            "tue": 1,
            "wed": 2,
            "thu": 3,
            "fri": 4,
            "sat": 5,
            "sun": 6,
        }

        for schedule in all_schedules:
            weekday_raw = schedule.weekday

            if isinstance(weekday_raw, int) or (
                isinstance(weekday_raw, str) and weekday_raw.isdigit()
            ):
                weekday_index = int(weekday_raw)
            else:
                weekday_index = SHORT_TO_INDEX[weekday_raw.strip().lower()]

            current_time = datetime.combine(datetime.min, schedule.opening_time)
            closing_time = datetime.combine(datetime.min, schedule.closing_time)
            while current_time < closing_time:
                all_times[calendar.day_name[weekday_index]].append(current_time.time())
                current_time += duration

        return all_times

    def is_overlap(self, slot_start, slot_end, appointment_start, appointment_end):
        """
        Retorna True se houver sobreposição entre os intervalos de tempo.
        """
        return not (slot_end <= appointment_start or slot_start >= appointment_end)

    def get_available_times(self, initial_date, final_date, type):
        type_map = {
            Appointment.IN_PERSON: ClinicOpeningSchedule.PRESENTIAL,
            Appointment.ONLINE: ClinicOpeningSchedule.ONLINE,
        }
        schedule_type = type_map.get(type, type)
        duration = (
            timedelta(minutes=self.clinic.online_clinic_duration_minutes)
            if schedule_type == ClinicOpeningSchedule.ONLINE
            else timedelta(minutes=self.clinic.duration_minutes)
        )

        all_available_times = self.get_all_times(schedule_type, duration)
        available_times = defaultdict(list)
        dt = initial_date

        while dt <= final_date:
            appointments = self.appointments.filter(date__date=dt).exclude(
                status=Appointment.CANCELED
            )
            weekday_name = calendar.day_name[dt.weekday()]
            available_times[dt.strftime("%Y-%m-%d")] = all_available_times.get(
                weekday_name, []
            ).copy()
            for appointment in appointments:
                appointment_duration = (
                    timedelta(minutes=self.clinic.duration_minutes)
                    if appointment.type == Appointment.IN_PERSON
                    else timedelta(minutes=self.clinic.online_clinic_duration_minutes)
                )
                appointment_date = timezone.localtime(appointment.date)
                appointment_start = datetime.combine(dt, appointment_date.time())
                appointment_end = appointment_start + appointment_duration
                available_times[dt.strftime("%Y-%m-%d")] = [
                    time_slot
                    for time_slot in available_times[dt.strftime("%Y-%m-%d")]
                    if not self.is_overlap(
                        datetime.combine(dt, time_slot),
                        datetime.combine(dt, time_slot) + duration,
                        appointment_start,
                        appointment_end,
                    )
                ]
            dt += timedelta(days=1)

        return available_times


class Clinic(BaseModel):
    id = HashedAutoField(primary_key=True)
    professional = models.OneToOneField(
        Professional,
        on_delete=models.PROTECT,
        related_name="clinic",
        verbose_name=_("Profissional"),
    )
    is_presential_clinic = models.BooleanField(
        default=True, verbose_name=_("Clínica Presencial")
    )
    is_online_clinic = models.BooleanField(
        default=False, verbose_name=_("Clínica Online")
    )

    # Presential clinic fields
    name = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Nome da Clínica")
    )
    zip_code = models.CharField(max_length=20, **NULLABLE, verbose_name=_("CEP"))
    registered_neighborhood = models.ForeignKey(
        "address.Neighborhood",
        on_delete=models.PROTECT,
        help_text=_("ID do bairro encontrado na API de busca de CEP"),
        **NULLABLE,
        verbose_name=_("ID do bairro"),
    )
    neighborhood = models.CharField(
        max_length=200,
        help_text=_("Nome do bairro"),
        **NULLABLE,
        verbose_name=_("Bairro"),
    )
    city = models.CharField(
        max_length=200,
        help_text=_("Nome da cidade"),
        **NULLABLE,
        verbose_name=_("Cidade"),
    )
    state = models.ForeignKey(
        "address.State", on_delete=models.PROTECT, verbose_name=_("Estado"), **NULLABLE
    )
    address = models.CharField(
        max_length=200,
        help_text=_("Endereço completo, incluindo número"),
        **NULLABLE,
        verbose_name=_("Endereço"),
    )
    address_line2 = models.CharField(
        max_length=200,
        help_text=_("Complemento do endereço"),
        **NULLABLE,
        verbose_name=_("Complemento"),
    )
    phone = PhoneNumberField(
        help_text=_("Telefone da clínica"),
        **NULLABLE,
        verbose_name=_("Telefone da clínica"),
    )
    phone_whatsapp = PhoneNumberField(
        help_text=_("Telefone WhatsApp"), **NULLABLE, verbose_name=_("Whatsapp")
    )
    consult_price = models.DecimalField(
        decimal_places=2,
        max_digits=9,
        validators=[MinValueValidator(Decimal(0.00))],
        help_text=_("Valor da Consulta"),
        **NULLABLE,
        verbose_name=_("Valor da Consulta"),
    )
    duration_minutes = models.PositiveIntegerField(
        validators=[MinValueValidator(1)],
        help_text=_("Duração em minutos da consulta"),
        **NULLABLE,
        verbose_name=_("Duração da consulta"),
    )
    accepts_insurance_providers = models.BooleanField(
        default=False,
        help_text=_("Aceita Convênios?"),
        verbose_name=_("Aceita Convênios"),
    )
    provides_accessibility_standards = models.BooleanField(
        default=False,
        help_text=_("Clínica possui acessibilidade?"),
        verbose_name=_("Clínica possui acessibilidade"),
    )

    # Online clinic fields
    online_clinic_phone = PhoneNumberField(
        help_text=_("Telefone da clínica"),
        **NULLABLE,
        verbose_name=_("Telefone da clínica"),
    )
    online_clinic_phone_whatsapp = PhoneNumberField(
        help_text=_("Telefone WhatsApp"), **NULLABLE, verbose_name="WhatsApp"
    )
    online_clinic_consult_price = models.DecimalField(
        decimal_places=2,
        max_digits=9,
        validators=[MinValueValidator(Decimal(0.00))],
        help_text=_("Valor da Consulta"),
        **NULLABLE,
        verbose_name=_("Valor da Consulta online"),
    )
    online_clinic_duration_minutes = models.PositiveIntegerField(
        validators=[MinValueValidator(1)],
        help_text=_("Duração em minutos da consulta"),
        **NULLABLE,
        verbose_name=_("Duração da consulta"),
    )
    online_clinic_accepts_insurance_providers = models.BooleanField(
        default=False,
        help_text=_("Aceita Convênios?"),
        verbose_name=_("Aceita Convênios"),
    )

    def __str__(self):
        return self.name or f"{self._meta.verbose_name} (id: {self.id})"

    class Meta:
        verbose_name = _("Clínica")
        verbose_name_plural = _("Clínicas")
        app_label = "lacreisaude"

    def post_create_instance(self, *args, **kwargs):
        watson.default_search_engine.update_obj_index(self.professional)

    def post_update_instance(self, *args, **kwargs):
        watson.default_search_engine.update_obj_index(self.professional)

    @property
    def full_address(self):
        parts = [
            self.address or "",
            self.address_line2 or "",
            self.neighborhood or "",
            self.city or "",
            self.state.code if self.state else "",
        ]
        return ", ".join(filter(None, parts)) or "Endereço não cadastrado"


class ClinicOpeningSchedule(BaseModel):
    ONLINE = "online"
    PRESENTIAL = "presential"
    SCHEDULE_TYPES = [
        (ONLINE, _("Online")),
        (PRESENTIAL, _("Presential")),
    ]
    SUN = "sun"
    MON = "mon"
    TUE = "tue"
    WED = "wed"
    THU = "thu"
    FRI = "fri"
    SAT = "sat"
    WEEKDAYS = [
        (SUN, _("Domingo")),
        (MON, _("Segunda-feira")),
        (TUE, _("Terça-feira")),
        (WED, _("Quarta-feira")),
        (THU, _("Quinta-feira")),
        (FRI, _("Sexta-feira")),
        (SAT, _("Sábado")),
    ]
    WEEKDAYS_MAP = {k: v for k, v in WEEKDAYS}
    id = HashedAutoField(primary_key=True)
    schedule_type = models.CharField(
        max_length=15,
        choices=SCHEDULE_TYPES,
        default="presential",
        verbose_name=_("Tipo de agendamento"),
    )
    weekday = models.CharField(
        max_length=3, choices=WEEKDAYS, verbose_name="Dia da semana"
    )
    clinic = models.ForeignKey(
        Clinic, on_delete=models.CASCADE, related_name="opening_schedules"
    )
    opening_time = models.TimeField(verbose_name="Horário de Abertura")
    closing_time = models.TimeField(verbose_name="Horário de fechamento")

    class Meta:
        verbose_name = _("Horário de abertura da clínica")
        verbose_name_plural = _("Horários de abertura da clínica")
        app_label = "lacreisaude"


class ProfessionalReview(BaseModel):
    id = HashedAutoField(primary_key=True)
    reviewed_by = models.ForeignKey(
        "lacreiid.User", on_delete=models.PROTECT, verbose_name=_("Revisado por")
    )
    professional = models.ForeignKey(
        Professional,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Profissional"),
    )
    professional_name = models.CharField(
        max_length=250, verbose_name=_("Nome do profissional")
    )
    profession = models.ForeignKey(
        Profession,
        on_delete=models.PROTECT,
        null=True,
        blank=True,
        verbose_name=_("Profissão"),
    )
    internal_note = models.TextField(verbose_name=_("Nota interna"))
    status = models.CharField(
        max_length=20,
        choices=[
            ("approved", "Aprovado"),
            ("rejected", "Rejeitado"),
        ],
    )
    rejected_professional_data = models.JSONField(
        null=True,
        blank=True,
    )
    step = models.CharField(
        max_length=30,
        choices=VERIFICATION_STEPS_CHOICES,
        null=True,
        verbose_name=_("Etapas"),
    )

    def __str__(self):
        return self.professional_name

    class Meta:
        verbose_name = _("Revisão de profissional")
        verbose_name_plural = _("Revisão de profissionais")
        app_label = "lacreisaude"

    def pre_create_instance(self, *args, **kwargs):
        self.professional_name = self.professional.full_name

    def save(self, *args, **kwargs):
        self.request = kwargs.pop("request") if "request" in kwargs else None
        return super().save(*args, **kwargs)

    def post_create_instance(self, *args, **kwargs):
        VerificationStepClass = VERIFICATION_STEPS_MAP[self.step]["verification_step"]
        verification_step = VerificationStepClass(self.professional)

        # Set approved or rejected from the verification step in the service
        getattr(verification_step, f"set_{self.status}")(self, self.request)


class ContactRequest(BaseModel):
    id = HashedAutoField(primary_key=True)
    requester = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        help_text=_("Pessoa usuária solicitando o contato da profissional"),
        on_delete=models.deletion.PROTECT,
        verbose_name=_("Solicitante"),
    )
    requester_ip_address = models.GenericIPAddressField(verbose_name=_("Endereço IP"))
    requester_phone_number = PhoneNumberField(verbose_name=_("Telefone solicitante"))
    requester_user_agent = models.TextField(verbose_name=_("User Agent"))
    professional = models.ForeignKey(
        Professional, on_delete=models.deletion.PROTECT, verbose_name=_("Profissional")
    )
    validation_code = models.CharField(
        max_length=6, verbose_name=_("Código de validação")
    )
    code_confirmed = models.BooleanField(
        default=False, verbose_name=_("Código confirmado")
    )
    expires_at = models.DateTimeField(verbose_name=_("Expira em"))
    created_at = models.DateTimeField(default=timezone.now, verbose_name=_("Criado em"))

    class Meta:
        verbose_name = _("Solicitação de Contato")
        verbose_name_plural = _("Solicitações de Contato")
        app_label = "lacreisaude"

    # Constants
    EXPIRES_IN_DAYS = 7

    # Manager
    objects = ContactRequestManager()

    def __str__(self) -> str:
        return f"{self.requester} -> {self.professional}"

    def pre_create_instance(self, *args, **kwargs):
        self.expires_at = timezone.now() + timedelta(
            days=ContactRequest.EXPIRES_IN_DAYS
        )
        self.validation_code = randint(100000, 999999)

    @property
    def sms_message(self):
        return CONTACT_REQUEST_SMS_MESSAGE.format(validation_code=self.validation_code)


class Complaint(BaseModel):
    id = HashedAutoField(primary_key=True)
    complaint_types = models.CharField(
        max_length=200, null=True, blank=True, verbose_name=_("Tipos de comportamento")
    )
    other_complaint_type = models.CharField(
        max_length=200,
        null=True,
        blank=True,
        verbose_name="Outros tipos de comportamento",
    )
    incident_date = models.DateField(verbose_name=_("Data do ocorrido"))
    incident_time = models.TimeField(verbose_name=_("Horário do ocorrido"))
    reported_by = models.ForeignKey(
        "lacreiid.User", on_delete=models.PROTECT, verbose_name=_("Reportado por")
    )
    description = models.TextField(verbose_name=_("Descrição do ocorrido"))
    professional = models.ForeignKey(
        Professional, on_delete=models.deletion.PROTECT, verbose_name=_("Profissional")
    )

    class Meta:
        verbose_name = _("Denúncia")
        verbose_name_plural = _("Denúncias")
        app_label = "lacreisaude"

    def post_create_instance(self, *args, **kwargs):
        context = NewComplaintContext(
            complaint__created_at=self.created_at,
            complaint__id=self.id,
            complaint_detail_url=f"{settings.API_HOST}{reverse('admin:lacreisaude_complaint_change', args=[self.id])}",
        ).model_dump(mode="json")

        notification.send(
            self,
            template_prefix="complaints/new_complaint",
            email=settings.COMPLAINT_EMAILS,
            context=context,
            recipient=None,
        )


class ProfessionalSubaccount(BaseModel):
    id = HashedAutoField(primary_key=True)
    professional = models.OneToOneField(
        "lacreisaude.Professional",
        on_delete=models.CASCADE,
        related_name="subaccount",
        verbose_name=_("Profissional"),
    )
    asaas_id = models.CharField(
        max_length=100, unique=True, verbose_name=_("ID no Asaas")
    )
    onboarding_url = models.URLField(
        max_length=500, **NULLABLE, verbose_name=_("URL de Onboarding de Documentos")
    )
    status_general = models.CharField(
        max_length=50, default="AWAITING_APPROVAL", verbose_name=_("Status Geral")
    )
    status_commercial_info = models.CharField(
        max_length=50,
        default="AWAITING_APPROVAL",
        verbose_name=_("Status Info. Comerciais"),
    )
    status_bank_account = models.CharField(
        max_length=50,
        default="AWAITING_APPROVAL",
        verbose_name=_("Status Conta Bancária"),
    )
    status_documentation = models.CharField(
        max_length=50,
        default="AWAITING_APPROVAL",
        verbose_name=_("Status Documentação"),
    )

    class Meta:
        verbose_name = _("Subconta de Profissional (Asaas)")
        verbose_name_plural = _("Subcontas de Profissionais (Asaas)")
        app_label = "lacreisaude"

    def __str__(self):
        return f"Subconta de {self.professional.full_name} ({self.asaas_id})"


class SubaccountDocument(BaseModel):
    """
    Armazena os arquivos de documento enviados para a verificação
    de uma subconta no Asaas.
    """

    id = HashedAutoField(primary_key=True)
    subaccount = models.ForeignKey(
        ProfessionalSubaccount,
        on_delete=models.CASCADE,
        related_name="documents",
        verbose_name=_("Subconta"),
    )
    document_type = models.CharField(
        max_length=100,
        verbose_name=_("Tipo de Documento"),
        help_text=_("Ex: IDENTIFICATION, SOCIAL_CONTRACT"),
    )
    file = models.FileField(
        upload_to=HashedFileName("subaccount_documents"),
        verbose_name=_("Arquivo do Documento"),
    )
    asaas_id = models.CharField(
        max_length=100, unique=True, verbose_name=_("ID do Documento no Asaas")
    )
    status = models.CharField(
        max_length=50, default="pending", verbose_name=_("Status do Documento")
    )
    rejection_reason = models.TextField(
        **NULLABLE, verbose_name=_("Motivo da Rejeição")
    )

    class Meta:
        verbose_name = _("Documento de Subconta")
        verbose_name_plural = _("Documentos de Subcontas")
        app_label = "lacreisaude"

    def __str__(self):
        return f"Documento {self.document_type} de {self.subaccount.professional.full_name}"



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class LacreiidConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "lacreiid"
    label = "lacreiid"
    verbose_name = _("Lacrei ID")



from django.utils.translation import gettext as _

PROFESSIONAL_ALREADY_APPROVED_FOR_STEP = _(
    "Pessoa profissional já foi approvada para esta etapa"
)

BOARD_VERIFICATION_NOT_APPROVED_YET = _(
    "Inclua uma validação do número conselho antes de prosseguir com esta etapa."
)


PROFESSIONAL_VALIDATION_PENDING_ADMIN = _("Pendente Lacrei")
PROFESSIONAL_VALIDATION_PENDING_DATA_INPUT = _("Pendente Profissional")



import logging

import requests
from requests.exceptions import RequestException, Timeout

logger = logging.getLogger("celery")


class BrasilAPIClient:
    BASE_URL = "https://brasilapi.com.br/api"

    def __init__(self, version: str = "v1", *args, **kwargs):
        self.version = version

    def get_banks(self):
        url = f"{self.BASE_URL}/banks/{self.version}/"
        try:
            response = requests.get(url, timeout=4)
            response.raise_for_status()
            data = response.json()
            return data if data else []
        except (RequestException, Timeout):
            logger.error("Erro ao buscar dados na API de bancos", exc_info=True)
            return []



from django.core.exceptions import ValidationError


class CannotSetStatusError(ValidationError):
    """
    Exception used when a transition can't be completed, like set_approved.
    """


class VerificatoinStepAlreadyApproved(CannotSetStatusError):
    pass


class VerificationStepConflict(CannotSetStatusError):
    pass
